"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _Node = _interopRequireDefault(require("./Node"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Block extends _Node.default {
  /**
   * @type {Array<Node|React.ReactNode>}
   */

  /**
   * @param {string} name
   * @param {Array<Node|React.ReactNode>} children
   */
  constructor(name, children = []) {
    super(name);

    _defineProperty(this, "children", void 0);

    this.children = children;
  }
  /**
   * Render the node
   * @param {function(name: string): (React.ComponentClass|React.ReactNode)} getter
   * @param {React.Key} [key]
   * @return {React.ReactNode}
   */


  render(getter, key) {
    const {
      name
    } = this;
    const Component = getter(name);

    if (typeof Component !== 'function') {
      throw new TypeError(`Renderer for '${name}' must be a function`);
    }

    return /*#__PURE__*/_react.default.createElement(Component, {
      key: key,
      name: name
    }, this.renderChildren(getter));
  }
  /**
   * Render children nodes
   * @param {function(name: string): (React.ComponentClass|React.ReactNode)} getter
   * @return {React.ReactNode}
   */


  renderChildren(getter) {
    const {
      children
    } = this;

    switch (children.length) {
      case 0:
        return null;

      case 1:
        const [child] = children;

        if (child instanceof _Node.default) {
          return child.render(getter);
        }

        return child;
    }

    return children.map((child, index) => {
      if (child instanceof _Node.default) {
        return child.render(getter, index);
      }

      return child;
    });
  }

}

exports.default = Block;